MODULE ST7735;
(* =========================================================================  
   Example ARM Cortex-M3 Oberon Module  
 
   Description:
     Sitronix ST7735 Display Controller Driver for 128x160 TFT display

   Target:
     STM32L152 systems
     
   Tested on: 
     AdaFruit 1.8" Colour TFT Shield PID: 802
     + STM Nucleo-64 L152RE Board
   
   Reference:
     Sitronix ST7735 262K Color Single-Chip TFT Controller/Driver
     v2.1 2010-02-01
     
   (c) 2015-2019 CFB Software
   http://www.astrobe.com
   
   ========================================================================= *)
IMPORT Error, Fonts, ResData, GPIO, Timers, MCU, SPI, SYSTEM;

CONST 
  SLPOUT  = 011H; (* Sleep out & booster on *)
  CASET   = 02AH; (* Column address set *)
  RASET   = 02BH; (* Row address set *)
  RAMWR   = 02CH; (* Writing to memory *)
  MADCTL  = 036H; (* Memory data access control *)
  COLMOD  = 03AH; (* Colour mode - interface pixel format *)
  DISPON  = 029H; (* Display on *)
  
  (* 16-bit RRRRR-GGGGGG-BBBBB color definitions *)
  Black*   = 00000H;
  Red*     = 0F800H;
  Green*   = 007E0H;
  Blue*    = 0001FH;
  White*   = Red + Green + Blue;
  Cyan*    = Green + Blue;
  Magenta* = Red + Blue;
  Yellow*  = Red + Green;
  
  MaxX* = 159;
  MaxY* = 127;
  
  (* 8 x 10 font *)
  MaxCol* = MaxX DIV 8;
  MaxRow* = MaxY DIV 10;

  Data = TRUE;
  Command = FALSE;
  Low = FALSE;
  High = TRUE;
  
  (* ========================================================================= *) 
  (*  Nucleo-64   Display
      ----        -------
      AVDD        VCC
      GND         Gnd 
      PB10   D06  RESET
      PA9    D08  D/C
      PA7    D11  MOSI1
      PA5    D13  SCK1
      PB6    D10  CS 
      PB4    D05  LITE
  *)

  PORTA = MCU.GPIOABase;
  PORTB = MCU.GPIOBBase;

VAR
  timer: Timers.Timer;
  pinBackLight, pinReset, pinCS, pinDC, pinSCK, pinMOSI: GPIO.Pin;
  font: ResData.Resource;
    
  (* ========================================================================= *) 
  PROCEDURE LoadFont*(name: ARRAY OF CHAR): BOOLEAN;
  BEGIN
    ResData.Open(font, name);
    RETURN ResData.Size(font) > 0 
  END LoadFont;
    
  (* ========================================================================= *) 
  PROCEDURE CharToFont(ch: CHAR; VAR fontCh1, fontCh2: INTEGER);
  VAR
    index: INTEGER;
  BEGIN
    IF ORD(ch) < ORD(" ") THEN ch := "." END;
    index := (ORD(ch) - ORD(" ")) * 2;
    ResData.GetInt(font, index, fontCh1);
    ResData.GetInt(font, index + 1, fontCh2)
  END CharToFont;
  
  (* ========================================================================= *) 
  PROCEDURE BackLight(state: BOOLEAN);
  BEGIN
    GPIO.Put(pinBackLight, state)
  END BackLight;
  
  (* ========================================================================= *) 
  PROCEDURE Reset(state: BOOLEAN);
  BEGIN
    GPIO.Put(pinReset, state)
  END Reset;
  
  (* ========================================================================= *) 
  PROCEDURE SetDataType(dataType: BOOLEAN);
  BEGIN
    GPIO.Put(pinDC, dataType)
  END SetDataType;
  
  (* ========================================================================= *) 
  PROCEDURE SendCommand(data: BYTE);
  BEGIN
    SetDataType(Command);
    SPI.SendByte(data);
    SetDataType(Data)
  END SendCommand;

  (* ========================================================================= *) 
  PROCEDURE SetY(y1, y2: INTEGER);
  BEGIN
    SendCommand(CASET);
    SPI.SendByte(0);
    SPI.SendByte(y1);
    SPI.SendByte(0);
    SPI.SendByte(y2);
  END SetY;
  
  (* ========================================================================= *) 
  PROCEDURE SetX(x1, x2: INTEGER);
  BEGIN
    SendCommand(RASET);
    SPI.SendByte(0);
    SPI.SendByte(x1);
    SPI.SendByte(0);
    SPI.SendByte(x2);
  END SetX;
  
  (* ========================================================================= *) 
  PROCEDURE DrawColour(colour: INTEGER);
  BEGIN
    SPI.SendByte(BFX(colour, 15, 8));
    SPI.SendByte(BFX(colour,  7, 0))
  END DrawColour;
  
  (* ************************************************************************************* *)
  (* Draws a dot in the specified color at the specified x and y coordinates *)
  (* color = 16-bit color value rrrrrggggggbbbbb *)
  PROCEDURE DrawDot*(colour, x, y: INTEGER);
  BEGIN
    IF (x <= MaxX) & (y <= MaxY) & (x >= 0) & (y >= 0) THEN 
      SetY(y, y);
      SetX(x, x);
      SendCommand(RAMWR); 
      DrawColour(colour)
    END 
  END DrawDot;

  (* ========================================================================= *) 
  PROCEDURE DrawRectangle*(colour, x, y, width, length: INTEGER);
  VAR
    i, count, colourWord: INTEGER;
  BEGIN
    SetY(y, y + length - 1);
    SetX(x, x + width - 1);
    SendCommand(RAMWR); 
    count := (width * length) DIV 2;
    FOR i := 0 TO count - 1 DO 
      DrawColour(colour);
      DrawColour(colour)
    END
  END DrawRectangle;

  (* ========================================================================= *) 
  PROCEDURE DrawChar*(colour: INTEGER; ch: CHAR; x1, y1: INTEGER);
  VAR
    i: INTEGER;
    x2, y2: INTEGER;
    font, font1, font2: INTEGER;
  BEGIN
    CharToFont(ch, font1, font2);
    x2 := x1 + Fonts.Width - 1;
    y2 := y1 + Fonts.Height - 1;
    ASSERT((x2 <= MaxX) & (y2 <= MaxY), Error.input);
    SetY(y1, y2);
    SetX(x1, x2);
    SendCommand(RAMWR);
    font := font1;
    FOR i := 0 TO Fonts.Width * Fonts.Height - 1 DO 
      IF i = 32 THEN font := LSL(font2, 16) END;
      IF font < 0 THEN 
        DrawColour(colour)
      ELSE 
        DrawColour(0)
      END;
      font := LSL(font, 1);
    END;
  END DrawChar;
  
  (* ========================================================================= *) 
  PROCEDURE ClearScreen*(colour: INTEGER);
  BEGIN
    DrawRectangle(colour, 0, 0, MaxX + 1, MaxY + 1)
  END ClearScreen;
  
  (* ========================================================================= *) 
  PROCEDURE ConfigurePins;
  VAR
    config: GPIO.PortConfiguration;
    x: SET;
  BEGIN
    (* Common configuration *)
    config.outputType := GPIO.Push_Pull;
    config.speed := GPIO.Speed_High;
    config.resistors := GPIO.Floating;

    (* enable GPIO clocks *)
    SYSTEM.GET(MCU.RCC_AHBENR, x);
    SYSTEM.PUT(MCU.RCC_AHBENR, x + 
      {MCU.RCC_AHBENR_GPIOA} +
      {MCU.RCC_AHBENR_GPIOB}
      );
    
    (* enable SPI1 *)
    SYSTEM.GET(MCU.RCC_APB2ENR, x);
    SYSTEM.PUT(MCU.RCC_APB2ENR, x + {MCU.RCC_APB2ENR_SPI1});
    (* GPIO output pins *)      
    config.mode := GPIO.Mode_Out;
    config.alternateFunction := GPIO.AF0;
    GPIO.Map(PORTB, 10, pinReset);
    GPIO.Map(PORTB,  4, pinBackLight);
    GPIO.Map(PORTA,  9, pinDC);
    GPIO.Configure(pinReset, config);
    GPIO.Configure(pinBackLight, config);
    GPIO.Configure(pinDC, config);
    
    (* configure SPI1 pins *)
    config.mode := GPIO.Mode_AF;
    config.alternateFunction := GPIO.AF5;     
    GPIO.Map(PORTA, 5, pinSCK); 
    GPIO.Map(PORTB, 6, pinCS);
    GPIO.Map(PORTA, 7, pinMOSI);
    GPIO.Configure(pinSCK, config); 
    GPIO.Configure(pinCS, config);
    GPIO.Configure(pinMOSI, config)

  END ConfigurePins;

  (* ========================================================================= *) 
  (* Initialise the Sirtronix ST7735 TFT Controller *)
  PROCEDURE Init*();
  BEGIN
    Timers.Init(timer, Timers.TIM2, Timers.MSecs);
    SPI.Init(SPI.SPI1, ConfigurePins); 

    Reset(FALSE);
    SetDataType(Command);
    Reset(TRUE);
    Timers.Delay(timer, 120); 
    SendCommand(SLPOUT); 
    Timers.Delay(timer, 120); 
    SendCommand(COLMOD); 
    SPI.SendByte(05H);   (* RGB16 5-6-5 *)
    SendCommand(MADCTL); (* Portrait RGB *)
    SPI.SendByte(046H); 
    SendCommand(DISPON); 
    BackLight(TRUE)
  END Init;
  
END ST7735.
